/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "SourcesCollection.h"

// LLVM
#ifdef LLVM_27_OR_28
#include <llvm/System/Path.h>
#else
#include <llvm/Support/Path.h>
#endif

#include <set>
#include <dirent.h>
#include <sys/types.h>

#include "GTLCore/String.h"
#include "GTLCore/Macros_p.h"
#include "GTLCore/Utils_p.h"

#include "Debug.h"
#include "Source.h"

using namespace OpenShiva;

static std::string getUserHomeDirectory() {
  char *h = getenv("HOME");
  return h ? h : "/";
}

struct SourcesCollection::Private {
  std::list< Source > kernels;
  void addDirectory( const std::string& path );
};


void SourcesCollection::Private::addDirectory( const std::string& path )
{
  SHIVA_DEBUG( "Looking for kernel in: " << path.c_str() );
  DIR *d = opendir(path.c_str());
  if(!d)
  {
    SHIVA_DEBUG( "Can't opendir " << path.c_str() << ": " << strerror(errno));
    return;
  }
  dirent *de;
  while(de = readdir(d))
  {
    std::string p = path + std::string("/") + std::string(de->d_name);
    SHIVA_DEBUG( "Kernel: " << p.c_str() << "?" );
    if( GTLCore::String( llvm::sys::path::extension(p) ).toLower() == ".shiva" )
    {
      SHIVA_DEBUG("Loading: " << p.c_str() );
      Source kernel;
      kernel.loadFromFile( p.c_str() );
      kernels.push_back( kernel );
    }
  }
  closedir(d);
}

SourcesCollection::SourcesCollection() : d(new Private)
{
  addDirectory( _OPENSHIVA_SHIVA_KERNELS_DIR_ );
  addDirectory( getUserHomeDirectory() + "/.OpenGTL/shiva/kernels" );
}

SourcesCollection::~SourcesCollection()
{
  delete d;
}

void SourcesCollection::addDirectory(const GTLCore::String& directory)
{
  d->addDirectory( (const std::string&)directory );
}

void SourcesCollection::registerSource( const Source& kernel )
{
  d->kernels.push_back( kernel );
}

std::list< Source> SourcesCollection::sources() const
{
  return d->kernels;
}

std::list< Source > SourcesCollection::sources(Source::SourceType _source) const
{
  std::list< Source > sources;
  foreach(const Source& source,d->kernels)
  {
    if(source.sourceType() == _source)
    {
      sources.push_back(source);
    }
  }
  return sources;
}

Source SourcesCollection::source(const GTLCore::String& _string) const
{
  foreach(Source source,d->kernels)
  {
    if(source.name() == _string)
    {
      return source;
    }
  }
  return Source();
}
